#!/bin/bash
#
# A script to install OpenIB kernal-modules
#

function usage {
    echo 'install.sh [--batch] [--help] [--tmpdir DIR] [-k|--ksrc DIR]'
    echo '           [--nosu] --clean'
    exit 0
}


# Execute command w/ echo and exit if it fail
ex()
{
    if ! "$@"; then
        echo "-E- $@"
        error "aborting ..."        
        exit 1
    fi
}

# Issue the error message given by $1 and exit with a non-zero
# exit code.

error() {
    echo "kernel-install.sh: *** $1" 1>&2
}

# Find the directory where this script resides
installdir=`cd ${0%*/*};pwd`
prog=$0
tmpdir=/tmp/ibgd-kernel
clean=no
batch=no
nosu=no
SET_ENV=1
vapi_ver=3.2
MAKE_SRP_TARGET=0

# Handle the case there is no makedepend...
export PATH=${PATH}:${installdir}

while [ "$1" ]; do
 	case "$1" in
		-t|--tmpdir)
                        tmpdir=$2
                        shift 2
                        ;;
                -k|--ksrc)
                        KSRC=$2
                        shift 2
                        ;;
                 --vapi)
                        vapi_ver=$2
                        shift 2
                        ;;
		-h|--help)
                        usage
                        shift
                        ;;
		-c|--clean)
                        clean=yes
                        shift
                        ;;
		-b|--batch)
                        batch=yes
                        shift
                        ;;
		--nosu)
                        nosu=yes
                        shift
                        ;;
                --build_only)
                        SET_ENV=0
                        shift
                        ;;        
 		--)
                        shift
                        break
                        ;;
 		*)
                        echo "Wrong parameter error!"
                        usage
                        exit 1
                        ;;
   esac
done

echo "Installing IBGD Kernel Modules"

if [ "$nosu" == "no" ]; then
    if [ $UID != 0 ]; then
        echo You must be root to install IBGD
        exit 1
    fi
fi


# make sure we have the tmpdir
/bin/rm -rf ${tmpdir}
mkdir -p ${tmpdir}

# Open the tar file into the tmp dir:
cd ${tmpdir}
ex tar -zxf ${installdir}/infiniband.tgz

# Modify osm_base.h with current change set and BUILD ID:
buildid=`cat ${installdir}/BUILD_ID`

# we rely on the fact we have opened the tar file so 
# have a local base dir:
if [ ! -d infiniband ]; then 
	 echo "Expected to find a sub-directory named 'infiniband' in current dir"
 	 echo "TAR file probably was not opened locally!"
	 exit 1
fi

export MAKELOG=make.log
export INSTALL_LOG=install.log
export EXPECTED_MAKELOG_SIZE_26=24000
export EXPECTED_MAKELOG_SIZE_DEP_24=330000
export EXPECTED_MAKELOG_SIZE_MODULES_24=140000

KVERSION=$(uname -r)
KVERSION_TWO_DIGITS=2$(echo ${KVERSION} | cut -d"." -f2)
KSRC=${KSRC:-/lib/modules/`uname -r`/build}
KERNEL_TREE_ROOT=${KSRC%*/*}
MAKE_KERNEL_DIR=${KSRC}

                                 
#Special case for SuSE9.1 and SlES9.0 build tree
if [ -d ${KERNEL_TREE_ROOT}/source/ ]; then
   KSRC=${KERNEL_TREE_ROOT}/source
   MAKE_KERNEL_DIR=${KERNEL_TREE_ROOT}/build
fi

MAKE_PARAMS=""

if [ "${KVERSION_TWO_DIGITS}" == "26" ]; then
	MAKE_PARAMS="V=1"
fi

# take care of the make files according to the kernel version
cd ${tmpdir}
find . -name Makefile -type l | xargs /bin/rm -f
/bin/rm -f infiniband/ulp/srp

# special case: SRP driver is not source-compatible between kernel 2.4
# and kernel 2.6.
ex mv infiniband/ulp/srp-$KVERSION_TWO_DIGITS  infiniband/ulp/srp
/bin/rm -rf infiniband/ulp/srp-*

# Check if SRP_target sources exists
if [ -e  ${installdir}/srp_target.tgz ]; then
    echo "Found SRP target sources"
    MAKE_SRP_TARGET=1
fi

# Integrate SRP target sources into the build tree
if [ $MAKE_SRP_TARGET -eq 1 ]; then
    cd ${tmpdir}
    ex tar -zxf ${installdir}/srp_target.tgz
    /bin/rm -rf infiniband/ulp/srp_target
    mkdir infiniband/ulp/srp_target
    cd infiniband/ulp/srp_target
    ex cp ${tmpdir}/srp_target/src/* .
    
    if [ "${KVERSION_TWO_DIGITS}" == "24" ]; then
        ex mv Makefile.24 Makefile
    else
        ex mv Makefile.26 Makefile
    fi

    cd ../../include
    ex cp ${tmpdir}/srp_target/files_to_create_patch/include/module* .
    
    cd ../ulp
    ex cp ${tmpdir}/srp_target/files_to_create_patch/ulp/Config.in .
    ex cp ${tmpdir}/srp_target/files_to_create_patch/ulp/Kconfig .

    if [ "${KVERSION_TWO_DIGITS}" == "24" ]; then
        ex cp ${tmpdir}/srp_target/files_to_create_patch/ulp/Makefile.24 Makefile.24
    else
        ex cp ${tmpdir}/srp_target/files_to_create_patch/ulp/Makefile.26 Makefile.26
    fi
fi

cd ${tmpdir}
MF=`find . -name Makefile.$KVERSION_TWO_DIGITS`

#fix this three lines next
for mf in $MF; do
    ex mv $mf `dirname $mf`/Makefile   
done
find . -name Makefile.2* -exec /bin/rm -f {} \;

/bin/rm -rf ${KSRC}/drivers/infiniband
if ! [ -d ${KSRC}/drivers ]; then
   ex mkdir -p ${KSRC}/drivers
fi
ex cp -rpf  ${tmpdir}/infiniband ${KSRC}/drivers/
##delete the un-needed vapi source tree
if [ "$vapi_ver" == "3.2" ]; then 
   ex rm -r  ${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-4.0
else 
   ex rm -r  ${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-3.2
fi

/bin/rm -f $MAKELOG
echo "Making IBGD kernel in ${tmpdir} ..."
##In 2.4 we need to make dep
# Define default EXTRAVERSION
EXTRAVERSION=$(uname -r | sed 's/^[0-9]\+\.[0-9]\+\.[0-9]\+//')

if [[ -f /etc/SuSE-release && "${KVERSION_TWO_DIGITS}" == "24" ]]; then 
  if ! ( ls -l /lib/modules/`uname -r`/build | grep include > /dev/null ); then
     EXTRAVERSION=""
  fi
fi

if [ -f /etc/rocks-release ]; then
  EXTRAVERSION=$(echo $EXTRAVERSION | sed 's/smp//')
fi

IS_FEDORA=0
if [ -f /etc/redhat-release ]; then
   if ( grep -w Fedora /etc/redhat-release > /dev/null ); then
	IS_FEDORA=1
   fi
fi

## Define modules configuration
#if [ $IS_FEDORA -eq 1 ]; then
## Disable SRP host and target compilation in Fedora Distribution
#MAKE_SRP_TARGET=0
#MODULES_CONF_PARAMS="CONFIG_INFINIBAND=m CONFIG_INFINIBAND_USER_CM=m CONFIG_INFINIBAND_IPOIB=m CONFIG_INFINIBAND_SDP=m CONFIG_INFINIBAND_DAPL_SRV=m CONFIG_INFINIBAND_KDAPL=m CONFIG_INFINIBAND_KDAPLTEST=m CONFIG_INFINIBAND_MELLANOX_HCA=m"
#else
MODULES_CONF_PARAMS="CONFIG_INFINIBAND=m CONFIG_INFINIBAND_USER_CM=m CONFIG_INFINIBAND_IPOIB=m CONFIG_INFINIBAND_SDP=m CONFIG_INFINIBAND_SRP=m CONFIG_INFINIBAND_DAPL_SRV=m CONFIG_INFINIBAND_KDAPL=m CONFIG_INFINIBAND_KDAPLTEST=m CONFIG_INFINIBAND_MELLANOX_HCA=m"
#fi

             
if [ $MAKE_SRP_TARGET -eq 1 ]; then
    MODULES_CONF_PARAMS="${MODULES_CONF_PARAMS} CONFIG_INFINIBAND_SRP_TARGET=m"
fi

if [ "${KVERSION_TWO_DIGITS}" == "24" ]; then
         #We need to make oldconfig only if it is not a new SuSE release or if .config does not exists
         if ! ( ls -l /lib/modules/`uname -r`/build | grep include > /dev/null ); then
             if ! [ -f /lib/modules/`uname -r`/build/.config ]; then
	        echo "Running Mrproper" 
                echo "make -C ${MAKE_KERNEL_DIR} mrproper" >> $MAKELOG
	        make -C ${MAKE_KERNEL_DIR} mrproper >> $MAKELOG
             fi
             echo "Making oldconfig for kernel 2.4 ..."
             echo "make -C ${MAKE_KERNEL_DIR} CONFIG_INFINIBAND=n oldconfig" >> $MAKELOG
             make -C ${MAKE_KERNEL_DIR} CONFIG_INFINIBAND=n oldconfig       >> $MAKELOG
             if [ $? != 0 ]; then 
      	      echo "Make failed. For more information look at:`pwd`/$MAKELOG"
      	      exit 1
             fi

             echo "Making dep for kernel 2.4 ..."
             
             echo "make -C ${MAKE_KERNEL_DIR} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} VAPI_VER=${vapi_ver} ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} dep" > tmp.log
             make -C ${MAKE_KERNEL_DIR} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} VAPI_VER=${vapi_ver} ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} dep >> tmp.log  2>&1 &
             make_pid=$!
             if [ "$batch" == "no" ]; then
                  ${installdir}/fprogbar.sh tmp.log $EXPECTED_MAKELOG_SIZE_DEP_24 $make_pid
             fi
             wait $make_pid >/dev/null 2>&1 
             if [ $? != 0 ]; then 
                cat tmp.log >> $MAKELOG
                /bin/rm -f tmp.log
	        echo "Make failed. For more information look at:`pwd`/$MAKELOG"
	        exit 1
             fi
             cat tmp.log >> $MAKELOG
             /bin/rm -f tmp.log
	 fi

         # Make dep with SUBDIRS
         echo "make -C ${MAKE_KERNEL_DIR} VAPI_VER=${vapi_ver} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} SUBDIRS=${KSRC}/drivers/infiniband ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} dep" >> $MAKELOG
         make -C ${MAKE_KERNEL_DIR} VAPI_VER=${vapi_ver} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} SUBDIRS=${KSRC}/drivers/infiniband ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} dep >> $MAKELOG 2>&1  
         if [ $? != 0 ]; then 
           echo "Make failed. For more information look at:`pwd`/$MAKELOG"
               exit 1
         fi
fi
echo "Making modules ..."

echo "make -C ${MAKE_KERNEL_DIR} VAPI_VER=${vapi_ver} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} SUBDIRS=${KSRC}/drivers/infiniband ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} ${MAKE_PARAMS} modules" > tmp.log
make -C ${MAKE_KERNEL_DIR} VAPI_VER=${vapi_ver} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} SUBDIRS=${KSRC}/drivers/infiniband ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} ${MAKE_PARAMS} modules >> tmp.log  2>&1  &

make_pid=$!
if [ "$batch" == "no" ]; then
  if [ "${KVERSION_TWO_DIGITS}" == "26" ]; then
          ${installdir}/fprogbar.sh tmp.log $EXPECTED_MAKELOG_SIZE_26 $make_pid
  else
          ${installdir}/fprogbar.sh tmp.log $EXPECTED_MAKELOG_SIZE_MODULES_24 $make_pid
  fi
fi

wait $make_pid >/dev/null 2>&1 

if [ $? != 0 ]; then 
         cat tmp.log >> $MAKELOG
         /bin/rm -f tmp.log
	 echo "Make failed. For more information look at:`pwd`/$MAKELOG"
	 exit 1
fi

cat tmp.log >> $MAKELOG
rm tmp.log

echo "Installing IBGD kernel in /lib/modules  ..."

echo "make -C ${MAKE_KERNEL_DIR} VAPI_VER=${vapi_ver} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} SUBDIRS=${KSRC}/drivers/infiniband ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} INSTALL_MOD_PATH=${tmpdir} ${MAKE_PARAMS} modules_install" >> $INSTALL_LOG
make -C ${MAKE_KERNEL_DIR} VAPI_VER=${vapi_ver} VAPI_KERNEL_SRC_DIR=${KSRC}/drivers/infiniband/hw/mellanox-hca/vapi-${vapi_ver} SUBDIRS=${KSRC}/drivers/infiniband ${MODULES_CONF_PARAMS} KERNELRELEASE=${KVERSION} EXTRAVERSION=${EXTRAVERSION} INSTALL_MOD_PATH=${tmpdir} ${MAKE_PARAMS} modules_install >> $INSTALL_LOG 2>&1  

if [ $? != 0 ]; then 
	 echo "Install failed. For more information look at:`pwd`/$INSTALL_LOG"
	 exit 1
fi

# Run this part to update running environment
if [ $SET_ENV -eq 1 ]; then
    if [ "${KVERSION_TWO_DIGITS}" == "24" ]; then
           /bin/rm -rf /lib/modules/${KVERSION}/kernel/drivers/ib
           /bin/rm -rf /lib/modules/${KVERSION}/kernel/drivers/infiniband
           mv ${tmpdir}/lib/modules/${KVERSION}/kernel/drivers/infiniband /lib/modules/${KVERSION}/kernel/drivers/ 
    else 
           /bin/rm -rf /lib/modules/${KVERSION}/kernel/drivers/ib
           /bin/rm -rf /lib/modules/${KVERSION}/kernel/drivers/infiniband
           /bin/rm -rf /lib/modules/${KVERSION}/extra/infiniband
	   mkdir -p /lib/modules/${KVERSION}/kernel/drivers/infiniband
           install -m 0644 ${tmpdir}/lib/modules/${KVERSION}/extra/*.ko /lib/modules/${KVERSION}/kernel/drivers/infiniband/
    fi
    
    if [ -f ${installdir}/BUILD_ID ]; then
        cp ${installdir}/BUILD_ID ${KSRC}/drivers/infiniband
    fi
    
    echo "Installation of IBGD kernel completed successfully"
    
fi

    if [ "${clean}" == "yes" ]; then
        echo "Cleaning up temporary build tree"
        /bin/rm -rf ${tmpdir}
    fi

exit 0
